function [ntt] = read_neuralynx_ntt(filename, begrecord, endrecord)

% READ_NEURALYNX_NTT reads a single tetrode file
%
% Use as
%   [ntt] = read_neuralynx_ntt(filename)

% Copyright (C) 2005, Robert Oostenveld
%
% $Log: read_neuralynx_ntt.m,v $
% Revision 1.2  2006/03/29 15:00:46  roboos
% changed some whitespace
%
% Revision 1.1  2005/05/20 06:39:35  roboos
% new implementation of single file readers
%

if nargin<2
  begrecord = 1;
end
if nargin<3
  endrecord = inf;
end

% The file starts with a 16*1024 bytes header in ascii, followed by a
% number of records (c.f. trials).

hdr = getheader(filename);

% The format of a tetrode record is
% int64 TimeStamp
% int32 ScNumber
% int32 CellNumber
% int32 Param[0]   int32 Param[7]
% int16 ChanW[0]
% int16 ChanX[0]
% int16 ChanY[0]
% int16 ChanZ[0]  
% int16 ChanW[31]
% int16 ChanX[31]
% int16 ChanY[31]
% int16 ChanZ[31]

fid = fopen(filename, 'rb', 'ieee-le');
fseek(fid, 0, 'eof');
filesize = ftell(fid);
fseek(fid, 16*1024, 'bof');                  % rewind, skip the header
fseek(fid, 304*(begrecord-1), 'cof');        % skip towards the first record to read
k = begrecord;
while ~feof(fid) && (ftell(fid)<filesize) && (k<=endrecord)
  i = k - begrecord + 1;
  k = k + 1;
  TimeStamp(i)  = fread(fid, 1, 'int64');
  ScNumber(i)   = fread(fid, 1, 'int32');
  CellNumber(i) = fread(fid, 1, 'int32');
  Param(i,:)    = fread(fid, 8, 'int32');
  buf           = fread(fid, [4 32], 'int16');
  if all(size(buf)==[4 32])
    dat{i} = buf;
  end
end
fclose(fid);

ntt.TimeStamp  = TimeStamp;
ntt.ScNumber   = ScNumber;
ntt.CellNumber = CellNumber;
ntt.Param      = Param;
ntt.dat        = dat;
ntt.hdr        = hdr;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for reading the header of a single channel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function hdr = getheader(filename);
fid     = fopen(filename, 'rb', 'ieee-le');
buf     = fread(fid, 16*1024, 'char');
buf     = buf(:)';
nl      = find(buf==10);    % determine the new-lines
cr      = find(buf==13);    % determine the carriage-returns
begchar = [1 nl(1:(end-1))];
endchar = nl - 1;
num     = length(nl);
hdr     = [];
for i=1:num
  line = fliplr(deblank(fliplr(deblank(char(buf(begchar(i):endchar(i)))))));
  if line(1)=='#'
    % line contains a comment
    continue
  else
    % strip the '-' sign
    if line(1)=='-'
      line = line(2:end);
    end
    % replace tabs with spaces
    line(find(line==9)) = ' ';
    % cut into pieces
    item = strread(line, '%s');
    if length(item)==2
      key = item{1};
      val = item{2};
      if any(val(1)=='-01234567989')
        % try to convert to number
        val = str2num(val);
        if isempty(val)
          % revert to the original text
          val = item{2};
        end
      end
      % assign the value to the header structure
      hdr = setfield(hdr, key, val);
    end
  end
end
fclose(fid);
